{% extends "admin/base_site.html" %}
{% load i18n %}

{% block breadcrumbs %}
<ol class="breadcrumbs">
<li><a href="{% url 'admin:index' %}">{% translate 'Home' %}</a></li>
<li aria-current="page">{% translate 'Logout' %}</li>
</ol>
{% endblock %}

{% block nav-sidebar %}{% endblock %}

{% block content %}

<p>{% translate "Thanks for spending some quality time with the web site today." %}</p>

<p><a href="{% url 'admin:index' %}">{% translate 'Log in again' %}</a></p>

{% endblock %}
