body {
    background: url("#foobar");
    background: url("http:foobar");
    background: url("https:foobar");
    background: url("data:foobar");
    background: url("chrome:foobar");
    background: url("//foobar");
    background: url();
}

/* @import url("non_exist.css") */

/* url("non_exist.png") */

/*

@import url("non_exist.css")

url("non_exist.png")

@import url(other.css)

*/

body {
    background: #d3d6d8 /*url("does.not.exist.png")*/ url(/static/cached/img/relative.png);
}

body {
    background: #d3d6d8 /* url("does.not.exist.png") */ url(/static/cached/img/relative.png) /*url("does.not.exist.either.png")*/;
}

body {
    content: "url(non_exist.png)";
    content: 'url(non_exist.png)';
}

/* Tailwind-style selector */
.tw\:bg-\[url\(\'non_exist.png\'\)\] { background: url(../img/relative.png); }
body { font-family: 'sans-serif'; }
