{% extends "admin/base_site.html" %}
{% load i18n static %}

{% block title %}{% if form.email.errors %}{% translate "Error:" %} {% endif %}{{ block.super }}{% endblock %}
{% block extrastyle %}
  {{ block.super }}
  <link rel="stylesheet" href="{% static "admin/css/widgets.css" %}" {% csp_nonce_attr %}>
  <link rel="stylesheet" href="{% static "admin/css/forms.css" %}" {% csp_nonce_attr %}>
{% endblock %}
{% block breadcrumbs %}
<ol class="breadcrumbs">
<li><a href="{% url 'admin:index' %}">{% translate 'Home' %}</a></li>
<li aria-current="page">{% translate 'Password reset' %}</li>
</ol>
{% endblock %}

{% block content %}

<p>{% translate 'Forgotten your password? Enter your email address below, and we’ll email instructions for setting a new one.' %}</p>

<form method="post">{% csrf_token %}
  <fieldset class="module aligned">
    <div class="form-row field-email">
      <div class="flex-container">
        <label for="id_email">{% translate 'Email address:' %}</label>
        {{ form.email.errors }}
        {{ form.email }}
      </div>
    </div>
  </fieldset>
  <div class="submit-row">
    <input type="submit" value="{% translate 'Reset my password' %}">
  </div>
</form>

{% endblock %}
